Module read

Module read 

Available on crate feature std only.
Read into uninitialized bytes logic.

Re-exports

pub use crate::extension_traits::VecExtendFromReader;

Modules

chainchain

Macros

auto_impl
Helper macro to alleviate the code duplication from implementing both Read and ReadIntoUninit.

Traits

ReadIntoUninit
Trait for a Readable type that can output the bytes read into uninitialised memory.