Available on crate feature
std only.Expand description
Read into uninitialized bytes logic.
Re-exports
pub use crate::extension_traits::VecExtendFromReader;
Modules
- chain
chain
Macros
- auto_
impl - Helper macro to alleviate the code duplication from implementing both
ReadandReadIntoUninit.
Traits
- Read
Into Uninit - Trait for a
Readable type that can output the bytes read into uninitialised memory.